@@ -1,6 +1,6 @@
-from uuid import uuid4
 from flask import request, g, current_app, Blueprint
 from rq import Connection, Queue
+from uuid import uuid4
 
 from .build import build
 from .common import get_request_hash
@@ -17,15 +17,28 @@ def get_distros() -> list:
     return ["openwrt"]
 
 
-@bp.route("/branches")
-def get_branches() -> dict:
-    if "branches" not in g:
-        g.branches = {}
-        for branch in current_app.config["BRANCHES"]:
-            if branch["enabled"]:
-                g.branches[branch["name"]] = branch
-        current_app.logger.info(f"Loaded {len(g.branches)} branches")
-    return g.branches
+@bp.route("/debug/get_versions")
+def get_versions() -> dict:
+    """Return available versions
+
+    The configuration stores a dict of versions containing additional
+    information like public signing key and upstream path.
+
+    Returns:
+        dict: latest available version per branch
+    """
+    if "versions" not in g:
+        g.versions = dict(
+            map(
+                lambda b: (b["name"], b),
+                filter(
+                    lambda b: b.get("enabled"),
+                    current_app.config["VERSIONS"]["branches"],
+                ),
+            )
+        )
+        current_app.logger.info(f"Loaded {len(g.versions)} versions")
+    return g.versions
 
 
 def get_redis():
@@ -39,29 +52,6 @@ def get_redis():
     return g.redis
 
 
-def get_latest() -> dict:
-    if "latest" not in g:
-        g.latest = list(
-            map(
-                lambda b: b["versions"][0],
-                filter(
-                    lambda b: b.get("enabled"),
-                    filter(
-                        lambda b: not b.get("snapshot"),
-                        current_app.config["BRANCHES"],
-                    ),
-                ),
-            )
-        )
-
-    return g.latest
-
-
-@bp.route("/latest")
-def api_latest():
-    return {"latest": get_latest()}
-
-
 def get_queue() -> Queue:
     """Return the current queue
 
@@ -74,140 +64,113 @@ def get_queue() -> Queue:
     return g.queue
 
 
-def validate_packages(req):
-    if "packages" in req and "packages_versions" in req:
-        return (
-            {
-                "status": "duplicate_package_list",
-                "message": "Use either `packages` or `packages_versions`, not both",
-            },
-            400,
-        )
-    elif req.get("packages"):
-        pass
-    elif req.get("packages_versions"):
-        req["packages"] = req["packages_versions"].keys()
-    else:
-        # no packages to check
-        return
-
-    req["packages"] = set(req["packages"]) - {"kernel", "libc", "libgcc"}
-
-    # store request packages temporary in Redis and create a diff
-    temp = str(uuid4())
-    r = get_redis()
-    pipeline = r.pipeline(True)
-    pipeline.sadd(temp, *set(map(lambda p: p.strip("-"), req["packages"])))
-    pipeline.expire(temp, 5)
-    pipeline.sdiff(
-        temp,
-        f"packages-{req['branch']}-{req['version']}-{req['target']}",
-        f"packages-{req['branch']}-{req['arch']}",
-    )
-    unknown_packages = list(map(lambda p: p.decode(), pipeline.execute()[-1]))
-
-    if unknown_packages:
-        return (
-            {
-                "status": "bad_packages",
-                "message": f"Unsupported package(s): {', '.join(unknown_packages)}",
-            },
-            422,
-        )
-
-
-def validate_request(req):
+def validate_request(request_data):
     """Validate an image request and return found errors with status code
 
     Instead of building every request it is first validated. This checks for
     existence of requested profile, distro, version and package.
 
     Args:
-        req (dict): The image request
+        request_data (dict): The image request
 
     Returns:
         (dict, int): Status message and code, empty if no error appears
 
     """
     for needed in ["version", "profile"]:
-        if needed not in req:
+        if needed not in request_data:
             return ({"status": "bad_request", "message": f"Missing {needed}"}, 400)
 
-    req["distro"] = req.get("distro", "openwrt")
-    if req["distro"] not in get_distros():
+    request_data["distro"] = request_data.get("distro", "openwrt").lower()
+    if request_data["distro"] not in get_distros():
         return (
             {
                 "status": "bad_distro",
-                "message": f"Unsupported distro: {req['distro']}",
+                "message": f"Unsupported distro: {request_data['distro']}",
             },
             400,
         )
 
-    req["version"] = req["version"]
-
-    if req["version"].count("-"):
-        # e.g. 21.02-snapshot
-        req["branch"] = req["version"].rsplit("-", maxsplit=1)[0]
-    else:
-        # e.g. snapshot or 19.07.7
-        req["branch"] = req["version"].rsplit(".", maxsplit=1)[0]
-
-    if req["branch"] not in get_branches().keys():
-        return (
-            {
-                "status": "bad_branch",
-                "message": f"Unsupported branch: {req['version']}",
-            },
-            400,
-        )
-
-    if req["version"] not in get_branches()[req["branch"]]["versions"]:
+    request_data["version"] = request_data["version"].lower()
+    request_data["branch"] = request_data["version"].rsplit(".", maxsplit=1)[0]
+    if request_data["branch"] not in get_versions().keys():
         return (
             {
                 "status": "bad_version",
-                "message": f"Unsupported version: {req['version']}",
+                "message": f"Unsupported version: {request_data['version']}",
+            },
+            400,
+        )
+    if request_data["version"] != get_versions()[request_data["branch"]][
+        "latest"
+    ] and not get_versions()[request_data["branch"]].get("support_legacy_versions"):
+        return (
+            {
+                "status": "legacy_version",
+                "message": "No legacy version support enabled",
             },
             400,
         )
 
-    r = get_redis()
+    # The supported_devices variable on devices uses a "," instead of "_"
+    # It is stored on device as board_name and send to the server for requests
+    # To be compatible with the build system profiles replace the "," with "_"
+    # TODO upstream request to store device profile on board
+    request_data["profile"] = request_data["profile"].replace(",", "_")
 
-    current_app.logger.debug("Profile before mapping " + req["profile"])
-
-    mapped_profile = r.hget(
-        f"mapping-{req['branch']}-{req['version']}",
-        req["profile"],
+    target = get_redis().hget(
+        f"profiles-{request_data['branch']}", request_data["profile"]
     )
-
-    if mapped_profile:
-        req["profile"] = mapped_profile.decode()
-
-    current_app.logger.debug("Profile after mapping " + req["profile"])
-
-    target = r.hget(
-        f"profiles-{req['branch']}-{req['version']}",
-        req["profile"],
-    )
-
     if not target:
         return (
             {
                 "status": "bad_profile",
-                "message": f"Unsupported profile: {req['profile']}",
+                "message": f"Unsupported profile: {request_data['profile']}",
             },
             400,
         )
+    else:
+        request_data["target"] = target.decode()
 
-    req["target"] = target.decode()
-    req["arch"] = get_branches()[req["branch"]]["targets"][req["target"]]
+    if request_data.get("packages"):
+        request_data["packages"] = set(request_data["packages"]) - {"kernel", "libc"}
 
-    package_problems = validate_packages(req)
-    if package_problems:
-        return package_problems
+        # store request packages temporary in Redis and create a diff
+        temp = str(uuid4())
+        pipeline = get_redis().pipeline(True)
+        pipeline.sadd(temp, *set(map(lambda p: p.strip("-"), request_data["packages"])))
+        pipeline.expire(temp, 5)
+        pipeline.sdiff(
+            temp,
+            f"packages-{request_data['branch']}",
+            f"packages-{request_data['branch']}-{request_data['target']}",
+        )
+        unknown_packages = list(map(lambda p: p.decode(), pipeline.execute()[-1]))
+
+        if unknown_packages:
+            return (
+                {
+                    "status": "bad_packages",
+                    "message": f"Unsupported package(s): {', '.join(unknown_packages)}",
+                },
+                422,
+            )
+    else:
+        request_data["packages"] = set()
 
     return ({}, None)
 
 
+@bp.route("/versions")
+def api_versions():
+    """API call to get available versions
+
+    Returns:
+        dict: Available versions in JSON format
+    """
+    return current_app.config["VERSIONS"]
+
+
 def return_job(job):
     """Return job status message and code
 
@@ -226,7 +189,7 @@ def return_job(job):
 
     elif job.is_queued or job.is_started:
         status = 202
-        response = {"status": job.get_status(), "queue_position": job.get_position()}
+        response = {"status": job.get_status()}
 
     elif job.is_finished:
         status = 200
@@ -280,36 +243,39 @@ def api_build():
     Retrns:
         (dict, int): Status message and code
     """
-    req = request.get_json()
-    current_app.logger.debug(f"req {req}")
-    if not req:
+    request_data = request.get_json()
+    current_app.logger.debug("rerequest_data {request_data}")
+    if not request_data:
         return {"status": "bad_request"}, 400
 
-    request_hash = get_request_hash(req)
+    request_hash = get_request_hash(request_data)
     job = get_queue().fetch_job(request_hash)
     response = {}
     status = 200
-    result_ttl = "24h"
-    failure_ttl = "12h"
+    if not current_app.config["DEBUG"]:
+        result_ttl = "24h"
+        failure_ttl = "12h"
+    else:
+        result_ttl = "15m"
+        failure_ttl = "15m"
 
     if job is None:
-        response, status = validate_request(req)
+        response, status = validate_request(request_data)
         if response:
             return response, status
 
-        req["store_path"] = current_app.config["STORE_PATH"]
-        req["cache_path"] = current_app.config["CACHE_PATH"]
-        req["upstream_url"] = current_app.config["UPSTREAM_URL"]
-        req["ca_pubkey"] = current_app.config["CA_PUBKEY"]
-        req["branch_data"] = get_branches()[req["branch"]]
+        request_data["store_path"] = current_app.config["STORE_PATH"]
+        request_data["cache_path"] = current_app.config["CACHE_PATH"]
+        request_data["upstream_url"] = current_app.config["UPSTREAM_URL"]
+        request_data["version_data"] = get_versions()[request_data["version"]]
 
         job = get_queue().enqueue(
             build,
-            req,
+            request_data,
             job_id=request_hash,
             result_ttl=result_ttl,
             failure_ttl=failure_ttl,
-            job_timeout="10m",
+            job_timeout="5m",
         )
 
     return return_job(job)
